@@ -8,6 +8,7 @@
 # This derived work is also licensed under the MIT license, see LICENSE.
 #++
 #
+require 'shellwords'
 
 raise "sorry, 1.8 was last decade" unless RUBY_VERSION >= '1.9'
 
@@ -18,6 +19,8 @@ Kramdown::Parser::Html::Constants::HTML_SPAN_ELEMENTS.concat my_span_elements
 
 require 'rexml/parsers/baseparser'
 require 'open3'                 # for math
+require 'json'                  # for math
+require 'rexml/document'        # for SVG and bibxml acrobatics
 
 class Object
   def deep_clone
@@ -37,16 +40,76 @@ module Kramdown
         @span_parsers.unshift(:iref)
       end
 
-      XREF_START = /\{\{(.*?)\}\}/u
+      SECTIONS_RE = /Section(?:s (?:[\w.]+, )*[\w.]+,? and)? [\w.]+/
+
+      def handle_bares(s, attr, format, href)
+        sa = s.sub(/\A\S+\s/, '').split(/,? and /)
+        sa[0..0] = *sa[0].split(', ')
+        sz = sa.size
+        if sz != 1         # we have to redo xml2rfc's work here
+          @tree.children << Element.new(:text, "Sections ", {}) # XXX needs to split into Section/Appendix
+          sa.each_with_index do |sec, i|
+            attr1 = {"target" => href, "section" => sec, "sectionFormat" => "bare"}
+            @tree.children << Element.new(:xref, nil, attr1)
+            text = if i == 0 && sz == 2
+                     " and "
+                   elsif i == sz-1
+                     " of "
+                   elsif i == sz-2
+                     ", and "
+                   else
+                     ", "
+                   end
+            @tree.children << Element.new(:text, text, {})
+          end
+          # attr stays unchanged, no section added
+        else
+          attr['section'] = sa[-1]
+          attr['sectionFormat'] = format
+        end
+      end
+
+      XREF_START = /\{\{(?:(?:\{(.*?)\}(?:\{(.*?)\})?)|(\X*?))((?:\}\})|\})/u
 
       # Introduce new {{target}} syntax for empty xrefs, which would
       # otherwise be an ugly ![!](target) or ![ ](target)
       # (I'd rather use [[target]], but that somehow clashes with links.)
       def parse_xref
         @src.pos += @src.matched_size
-        href = @src[1]
-        href = href.gsub(/\A[0-9]/) { "_#{$&}" } # can't start an IDREF with a number
-        el = Element.new(:xref, nil, {'target' => href})
+        unless @src[4] == "}}"
+          warn "*** #{@src[0]}: unmatched braces #{@src[4].inspect}"
+        end
+        if contact_name = @src[1]
+          attr = {'fullname' => contact_name}
+          if ascii_name = @src[2]
+            attr["asciiFullname"] = ascii_name
+          end
+          el = Element.new(:contact, nil, attr)
+        else
+          href = @src[3]
+          attr = {}
+          if $options.v3
+            # match Section ... of ...; set section, sectionFormat
+            case href.gsub(/[\u00A0\s]+/, ' ') # may need nbsp and/or newlines
+            when /\A(#{SECTIONS_RE}) of (.*)\z/
+              href = $2
+              handle_bares($1, attr, "of", href)
+            when /\A(.*), (#{SECTIONS_RE})\z/
+              href = $1
+              handle_bares($2, attr, "comma", href)
+            when /\A(.*) \((#{SECTIONS_RE})\)\z/
+              href = $1
+              handle_bares($2, attr, "parens", href)
+            when /\A([\w.]+)<(.*)\z/
+              href = $2
+              attr['section'] = $1
+              attr['sectionFormat'] = 'bare'
+            end
+          end
+          href = href.gsub(/\A[0-9]/) { "_#{$&}" } # can't start an IDREF with a number
+          attr['target'] = href
+          el = Element.new(:xref, nil, attr)
+        end
         @tree.children << el
       end
       define_parser(:xref, XREF_START, '{{')
@@ -66,6 +129,22 @@ module Kramdown
   end
 
   class Element
+
+    # Not fixing studly element names postalLine and seriesInfo yet
+
+    # occasionally regenerate the studly attribute name list via
+    # script in data/studly.rb
+    STUDLY_ATTR = %w(
+    asciiAbbrev asciiFullname asciiInitials asciiName asciiSurname
+    asciiValue blankLines derivedAnchor derivedContent derivedCounter
+    derivedLink displayFormat docName expiresDate hangIndent hangText
+    indexInclude iprExtract keepWithNext keepWithPrevious originalSrc
+    prepTime quoteTitle quotedFrom removeInRFC sectionFormat seriesNo
+    showOnFrontPage slugifiedName sortRefs submissionType symRefs tocDepth
+    tocInclude
+    )
+    STUDLY_ATTR_MAP = Hash[STUDLY_ATTR.map {|s| [s.downcase, s]}]
+
     def rfc2629_fix
       if a = attr
         if anchor = a.delete('id')
@@ -74,6 +153,11 @@ module Kramdown
         if anchor = a.delete('href')
           a['target'] = anchor
         end
+        attr.keys.each do |k|
+          if (d = k.gsub(/_(.|$)/) { $1.upcase }) != k or d = STUDLY_ATTR_MAP[k]
+            a[d] = a.delete(k)
+          end
+        end
       end
     end
   end
@@ -95,6 +179,18 @@ module Kramdown
 
       # :stopdoc:
 
+      KRAMDOWN_PERSISTENT = ENV["KRAMDOWN_PERSISTENT"]
+      KRAMDOWN_PERSISTENT_VERBOSE = /v/ === KRAMDOWN_PERSISTENT
+
+      if KRAMDOWN_PERSISTENT
+        begin
+          require 'net/http/persistent'
+          $http = Net::HTTP::Persistent.new name: 'kramdown-rfc'
+        rescue Exception => e
+          warn "** Can't set up persistent HTTP -- #{e}"
+        end
+      end
+
       # Defines the amount of indentation used when nesting XML tags.
       INDENTATION = 2
 
@@ -198,7 +294,6 @@ COLORS
       end
 
       def svg_clean(s)          # expensive, risky
-        require "rexml/document"
         d = REXML::Document.new(s)
         REXML::XPath.each(d.root, "//*[@shape-rendering]") { |x| x.attributes["shape-rendering"] = nil }  #; warn x.inspect }
         REXML::XPath.each(d.root, "//*[@text-rendering]") { |x| x.attributes["text-rendering"] = nil }  #; warn x.inspect  }
@@ -206,7 +301,12 @@ COLORS
         REXML::XPath.each(d.root, "//*[@fill]") { |x| x.attributes["fill"] = svg_munch_color(x.attributes["fill"], true) }
         REXML::XPath.each(d.root, "//*[@id]") { |x| x.attributes["id"] = svg_munch_id(x.attributes["id"]) }
 ##      REXML::XPath.each(d.root, "//rect") { |x| x.attributes["style"] = "fill:none;stroke:black;stroke-width:1" unless x.attributes["style"] }
+        # Fix for mermaid:
+        REXML::XPath.each(d.root, "//polygon") { |x| x.attributes["rx"] = nil; x.attributes["ry"] = nil }
         d.to_s
+      rescue => detail
+        warn "*** Can't clean SVG: #{detail}"
+        d
       end
 
       def memoize(meth, *args)
@@ -224,6 +324,14 @@ COLORS
         out
       end
 
+      def capture_croak(t, err)
+        if err != ''
+          err.lines do |l|
+            warn "*** [#{t}:] #{l.chomp}"
+          end
+        end
+      end
+
       def svg_tool_process(t, result)
         require 'tempfile'
         file = Tempfile.new("kramdown-rfc")
@@ -231,32 +339,46 @@ COLORS
         file.close
         case t
         when "goat"
-          result1, _s = Open3.capture2("goat #{file.path}", stdin_data: result);
+          result1, err, _s = Open3.capture3("goat #{file.path}", stdin_data: result);
         when "ditaa"        # XXX: This needs some form of option-setting
-          result1, _s = Open3.capture2("ditaa #{file.path} --svg -o -", stdin_data: result);
+          result1, err, _s = Open3.capture3("ditaa #{file.path} --svg -o -", stdin_data: result);
         when "mscgen"
-          result1, _s = Open3.capture2("mscgen -T svg -i #{file.path} -o -", stdin_data: result);
+          result1, err, _s = Open3.capture3("mscgen -T svg -i #{file.path} -o -", stdin_data: result);
         when "mermaid"
-          result1, _s = Open3.capture2("mmdc -i #{file.path}", stdin_data: result); #  -b transparent
+          result1, err, _s = Open3.capture3("mmdc -i #{file.path}", stdin_data: result); #  -b transparent
           outpath = file.path + ".svg"
-          result1 = File.read(outpath)
-          File.unlink(outpath)
+          result1 = File.read(outpath) rescue '' # don't die before providing error message
+          File.unlink(outpath) rescue nil        # ditto
         when "plantuml", "plantuml-utxt"
           plantuml = "@startuml\n#{result}\n@enduml"
-          result1, _s = Open3.capture2("plantuml -pipe -tsvg", stdin_data: plantuml);
-          result, _s = Open3.capture2("plantuml -pipe -tutxt", stdin_data: plantuml) if t == "plantuml-utxt"
+          result1, err, _s = Open3.capture3("plantuml -pipe -tsvg", stdin_data: plantuml);
+          result, err1, _s = Open3.capture3("plantuml -pipe -tutxt", stdin_data: plantuml) if t == "plantuml-utxt"
+          err << err1.to_s
+        when "math"
+          result1, err, _s = Open3.capture3("tex2svg --font STIX --speech=false #{Shellwords.escape(' ' << result)}");
+          result, err1, _s = Open3.capture3("asciitex -f #{file.path}")
+          err << err1
         end
+        capture_croak(t, err)
         # warn ["goat:", result1.inspect]
         file.unlink
         result1 = svg_clean(result1) unless t == "goat"
-        result1, _s = Open3.capture2("svgcheck -qa", stdin_data: result1);
+        result1, err, _s = Open3.capture3("svgcheck -qa", stdin_data: result1);
+        capture_croak("svgcheck", err)
         # warn ["svgcheck:", result1.inspect]
-        [result, result1]
+        if result1 == ''
+          warn "*** could not create svg for #{result.inspect[0...20]}..."
+          exit 65 # EX_DATAERR
+        end
+        [result, result1]       # text, svg
       end
 
+      ARTWORK_TYPES = %w(ascii-art binary-art call-flow hex-dump svg)
+
       def convert_codeblock(el, indent, opts)
         # el.attr['anchor'] ||= saner_generate_id(el.value) -- no longer in 1.0.6
         result = el.value
+        gi = el.attr.delete('gi')
         blockclass = el.attr.delete('class')
         if blockclass == 'language-tbreak'
           result = result.lines.map {|line| [line.chomp, 0]}
@@ -283,7 +405,7 @@ COLORS
               if md = cl.match(/\Alanguage-(.*)/)
                 t = artwork_attr["type"] = md[1] # XXX overwrite
               else
-                $stderr.puts "*** Unimplemented block class: #{cl}"
+                $stderr.puts "*** Unimplemented codeblock class: #{cl}"
               end
             end
           end
@@ -292,17 +414,27 @@ COLORS
             result[0,0] = "\n" unless result[0,1] == "\n"
           end
           el.attr.each do |k, v|
-            if md = k.match(/\Aartwork-(.*)/)
+            if md = k.match(/\A(?:artwork|sourcecode)-(.*)/)
               el.attr.delete(k)
               artwork_attr[md[1]] = v
             end
           end
           case t
-          when "goat", "ditaa", "mscgen", "plantuml", "plantuml-utxt", "mermaid"
+          when "goat", "ditaa", "mscgen", "plantuml", "plantuml-utxt", "mermaid", "math"
+            if gi
+              warn "*** Can't set GI #{gi} for composite SVG artset"
+            end
             result, result1 = memoize(:svg_tool_process, t, result)
             "#{' '*indent}<figure#{el_html_attributes(el)}><artset><artwork #{html_attributes(artwork_attr.merge("type"=> "svg"))}>#{result1.sub(/.*?<svg/m, "<svg")}</artwork><artwork #{html_attributes(artwork_attr.merge("type"=> "ascii-art"))}><![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]></artwork></artset></figure>\n"
           else
-            "#{' '*indent}<figure#{el_html_attributes(el)}><artwork#{html_attributes(artwork_attr)}><![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]></artwork></figure>\n"
+            gi ||= (
+              if !$options.v3 || !t || ARTWORK_TYPES.include?(t) || artwork_attr["align"]
+                "artwork"
+              else
+                "sourcecode"
+              end
+            )
+            "#{' '*indent}<figure#{el_html_attributes(el)}><#{gi}#{html_attributes(artwork_attr)}><![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]></#{gi}></figure>\n"
           end
         end
       end
@@ -345,8 +477,15 @@ COLORS
         if options[:auto_ids] && !el.attr['anchor']
           el.attr['anchor'] = saner_generate_id(el.options[:raw_text])
         end
+        if $options.v3
+          if sl = el.attr.delete('slugifiedName') # could do general name- play
+            attrstring = html_attributes({'slugifiedName' => sl})
+          end
+          irefs = "<name#{attrstring}>#{inner(el, indent, opts)}</name>" #
+        else
         clean, irefs = clean_pcdata(inner_a(el, indent, opts))
         el.attr['title'] = clean
+        end
         "#{end_sections(el.options[:level], indent)}#{' '*indent}<section#{@sec_level += 1; el_html_attributes(el)}>#{irefs}\n"
       end
 
@@ -366,7 +505,21 @@ COLORS
         end
       end
       alias :convert_ol :convert_ul
-      alias :convert_dl :convert_ul
+
+      def convert_dl(el, indent, opts)
+        if $options.v3
+          if hangindent = el.attr.delete('hangIndent')
+            el.attr['indent'] ||= hangindent # new attribute name wins
+          end
+          vspace = el.attr.delete('vspace')
+          if vspace && !el.attr['newline']
+            el.attr['newline'] = 'true'
+          end
+          "#{' '*indent}<dl#{el_html_attributes(el)}>\n#{inner(el, indent, opts.dup)}#{' '*indent}</dl>\n"
+        else
+          convert_ul(el, indent, opts)
+        end
+      end
 
       def convert_li(el, indent, opts)
         res_a = inner_a(el, indent, opts)
@@ -381,7 +534,16 @@ COLORS
         end
         "#{' '*indent}<t#{el_html_attributes(el)}>#{res}#{(res =~ /\n\Z/ ? ' '*indent : '')}</t>\n"
       end
+
       def convert_dd(el, indent, opts)
+        if $options.v3
+          out = ''
+          if !opts[:haddt]
+            out ="#{' '*indent}<dt/>\n" # you can't make this one up
+          end
+          opts[:haddt] = false
+          out << "#{' '*indent}<dd#{el_html_attributes(el)}>\n#{inner(el, indent, opts)}#{' '*indent}</dd>\n"
+        else
         output = ' '*indent
         if @in_dt == 1
           @in_dt = 0
@@ -395,15 +557,25 @@ COLORS
 #          output << "\n" << res << ' '*indent
 #        end
         output << "</t>\n"
+        end
       end
 
       def convert_dt(el, indent, opts) # SERIOUSLY BAD HACK:
+        if $options.v3
+          out = ''
+          if opts[:haddt]
+            out ="#{' '*indent}<dd><t/></dd>\n" # you can't make this one up
+          end
+          opts[:haddt] = true
+          out << "#{' '*indent}<dt#{el_html_attributes(el)}>\n#{inner(el, indent, opts)}#{' '*indent}</dt>\n"
+        else
         close = "#{' '*indent}</t>\n" * @in_dt
         @in_dt = 1
         vspace = opts[:vspace]
         vspaceel = "<vspace blankLines='#{vspace}'/>" if vspace
         ht = escape_html(inner(el, indent, opts), :attribute) # XXX this may leave gunk
-        "#{close}#{' '*indent}<t#{el_html_attributes(el)} hangText='#{ht}'>#{vspaceel}\n"
+        "#{close}#{' '*indent}<t#{el_html_attributes(el)} hangText=\"#{ht}\">#{vspaceel}\n"
+        end
       end
 
       HTML_TAGS_WITH_BODY=['div', 'script']
@@ -524,6 +696,10 @@ COLORS
         end
       end
 
+      def convert_contact(el, indent, opts)
+        "<contact#{el_html_attributes(el)}/>"
+      end
+
       REFCACHEDIR = ENV["KRAMDOWN_REFCACHEDIR"] || ".refcache"
 
       # warn "*** REFCACHEDIR #{REFCACHEDIR}"
@@ -531,6 +707,34 @@ COLORS
       KRAMDOWN_OFFLINE = ENV["KRAMDOWN_OFFLINE"]
       KRAMDOWN_REFCACHE_REFETCH = ENV["KRAMDOWN_REFCACHE_REFETCH"]
 
+      def get_and_write_resource(url, fn)
+        options = {}
+        if ENV["KRAMDOWN_DONT_VERIFY_HTTPS"]
+          options[:ssl_verify_mode] = OpenSSL::SSL::VERIFY_NONE
+        end             # workaround for OpenSSL on Windows...
+        # URI.open(url, **options) do |uf|          # not portable to older versions
+        OpenURI.open_uri(url, **options) do |uf|
+          s = uf.read
+          if uf.status[0] != "200"
+            warn "*** Status code #{status} while fetching #{url}"
+          else
+            File.write(fn, s)
+          end
+        end
+      end
+
+      def get_and_write_resource_persistently(url, fn)
+        t1 = Time.now
+        response = $http.request(URI(url))
+        if response.code != "200"
+          raise "Status code #{response.code} while fetching #{url}"
+        else
+          File.write(fn, response.body)
+        end
+        t2 = Time.now
+        warn "(#{"%.3f" % (t2 - t1)} s)" if KRAMDOWN_PERSISTENT_VERBOSE
+      end
+
       # this is now slightly dangerous as multiple urls could map to the same cachefile
       def get_and_cache_resource(url, cachefile, tvalid = 7200, tn = Time.now)
         fn = "#{REFCACHEDIR}/#{cachefile}"
@@ -544,7 +748,7 @@ COLORS
             message = "fetching"
             fetch_timeout = 60 # seconds; long timeout needed for Travis
           end
-          $stderr.puts "#{fn}: #{message}"
+          $stderr.puts "#{fn}: #{message} from #{url}"
           if ENV["HAVE_WGET"]
             `cd #{REFCACHEDIR}; wget -t 3 -T #{fetch_timeout} -Nnv "#{url}"` # ignore errors if offline (hack)
             begin
@@ -559,18 +763,15 @@ COLORS
             require 'timeout'
             begin
               Timeout::timeout(fetch_timeout) do
-                options = {}
-                if ENV["KRAMDOWN_DONT_VERIFY_HTTPS"]
-                  options[:ssl_verify_mode] = OpenSSL::SSL::VERIFY_NONE
-                end             # workaround for OpenSSL on Windows...
-#               URI.open(url, **options) do |uf|          # not portable to older versions
-                OpenURI.open_uri(url, **options) do |uf|
-                  s = uf.read
-                  if uf.status[0] != "200"
-                    warn "*** Status code #{status} while fetching #{url}"
-                  else
-                    File.write(fn, s)
+                if $http
+                  begin         # belt and suspenders
+                    get_and_write_resource_persistently(url, fn)
+                  rescue Exception => e
+                    warn "*** Can't get with persistent HTTP: #{e}"
+                    get_and_write_resource(url, fn)
                   end
+                else
+                  get_and_write_resource(url, fn)
                 end
               end
             rescue OpenURI::HTTPError, Errno::EHOSTUNREACH, Errno::ECONNREFUSED,
@@ -588,8 +789,12 @@ COLORS
 
       # [subdirectory name, cache ttl in seconds, does it provide for ?anchor=]
       XML_RESOURCE_ORG_MAP = {
-        "RFC" => ["bibxml", 86400*7], # these should change rarely
-        "I-D" => "bibxml3",
+        "RFC" => ["bibxml", 86400*7, false,
+                  ->(fn, n){ "https://www.rfc-editor.org/refs/bibxml/#{fn}"}
+                 ],
+        "I-D" => ["bibxml3", false, false,
+                  ->(fn, n){ "https://datatracker.ietf.org/doc/bibxml3/draft-#{n.sub(/\Adraft-/, '')}/xml" }
+                 ],
         "W3C" => "bibxml4",
         "3GPP" => "bibxml5",
         "ANSI" => "bibxml2",
@@ -610,9 +815,12 @@ COLORS
       XML_RESOURCE_ORG_HOST = ENV["XML_RESOURCE_ORG_HOST"] || "xml2rfc.tools.ietf.org"
       XML_RESOURCE_ORG_PREFIX = ENV["XML_RESOURCE_ORG_PREFIX"] ||
                                 "https://#{XML_RESOURCE_ORG_HOST}/public/rfc"
+      KRAMDOWN_USE_TOOLS_SERVER = ENV["KRAMDOWN_USE_TOOLS_SERVER"]
 
       KRAMDOWN_REFCACHETTL = (e = ENV["KRAMDOWN_REFCACHETTL"]) ? e.to_i : 3600
 
+      KRAMDOWN_NO_TARGETS = ENV['KRAMDOWN_NO_TARGETS']
+
       def convert_img(el, indent, opts) # misuse the tag!
         if a = el.attr
           alt = a.delete('alt').strip
@@ -632,18 +840,46 @@ COLORS
           to_insert = ""
           src.scan(/(W3C|3GPP|[A-Z-]+)[.]?([A-Za-z_0-9.\/\+-]+)/) do |t, n|
             fn = "reference.#{t}.#{n}.xml"
-            sub, ttl, can_anchor = XML_RESOURCE_ORG_MAP[t]
+            sub, ttl, _can_anchor, altproc = XML_RESOURCE_ORG_MAP[t]
             ttl ||= KRAMDOWN_REFCACHETTL  # everything but RFCs might change a lot
             puts "*** Huh: #{fn}" unless sub
-            url = "#{XML_RESOURCE_ORG_PREFIX}/#{sub}/#{fn}"
-            if can_anchor # create anchor server-side for stand_alone: false
-              url << "?anchor=#{anchor}"
-              fn[/.xml$/] = "--anchor=#{anchor}.xml"
+            if altproc && !KRAMDOWN_USE_TOOLS_SERVER
+              url = altproc.call(fn, n)
+            else
+              url = "#{XML_RESOURCE_ORG_PREFIX}/#{sub}/#{fn}"
             end
+            # if can_anchor # create anchor server-side for stand_alone: false
+            #   url << "?anchor=#{anchor}"
+            #   fn[/.xml$/] = "--anchor=#{anchor}.xml"
+            # end
             to_insert = get_and_cache_resource(url, fn.gsub('/', '_'), ttl)
             to_insert.scrub! rescue nil # only do this for Ruby >= 2.1
+
+            begin
+              d = REXML::Document.new(to_insert)
+              d.xml_decl.nowrite
+              d.root.attributes["anchor"] = anchor
+              if t == "RFC" or t == "I-D"
+                if KRAMDOWN_NO_TARGETS
+                  d.root.attributes["target"] = nil
+                  REXML::XPath.each(d.root, "/reference/format") { |x|
+                    d.root.delete_element(x)
+                  }
+                else
+                  REXML::XPath.each(d.root, "/reference/format") { |x|
+                    x.attributes["target"].sub!(%r{https?://www.ietf.org/internet-drafts/}, 
+                                                %{https://www.ietf.org/archive/id/}) if t == "I-D"
+                  }
+                end
+              end
+              to_insert = d.to_s
+            rescue Exception => e
+              warn "** Can't manipulate reference XML: #{e}"
+              broken = true
+              to_insert = nil
+            end
             # this may be a bit controversial: Don't break the build if reference is broken
-            if KRAMDOWN_OFFLINE
+            if KRAMDOWN_OFFLINE || broken
               unless to_insert
                 to_insert = "<reference anchor='#{anchor}'> <front> <title>*** BROKEN REFERENCE ***</title> <author> <organization/> </author> <date/> </front> </reference>"
                 warn "*** KRAMDOWN_OFFLINE: Inserting broken reference for #{fn}"
@@ -652,8 +888,7 @@ COLORS
               exit 66 unless to_insert # EX_NOINPUT
             end
           end
-          to_insert.sub(/<\?xml version=["']1.0["'] encoding=["']UTF-8["']\?>/, '')
-            .sub(/\banchor=(?:"[^"]+"|'[^']+')/, "anchor=\"#{anchor}\"")
+          to_insert
         else
           "<xref#{el_html_attributes(el)}>#{alt}</xref>"
         end
@@ -690,9 +925,14 @@ COLORS
       EMPH = { em: "emph", strong: "strong"}
 
       def convert_em(el, indent, opts)
+        if $options.v3
+          gi = el.type
+          "<#{gi}#{el_html_attributes(el)}>#{inner(el, indent, opts)}</#{gi}>"
+        else
         attrstring = el_html_attributes_with(el, {"style" => EMPH[el.type]})
         span, irefs = clean_pcdata(inner_a(el, indent, opts))
         "<spanx#{attrstring}>#{span}</spanx>#{irefs}"
+        end
       end
       alias :convert_strong :convert_em
 
@@ -710,13 +950,34 @@ COLORS
         :raquo => [::Kramdown::Utils::Entities.entity('raquo')]
       }
       def convert_typographic_sym(el, indent, opts)
-        TYPOGRAPHIC_SYMS[el.value].map {|e| entity_to_str(e)}.join('')
+        if (result = @options[:typographic_symbols][el.value])
+          escape_html(result, :text)
+        else
+          TYPOGRAPHIC_SYMS[el.value].map {|e| entity_to_str(e) }.join('')
+        end
       end
 
       def convert_smart_quote(el, indent, opts)
         entity_to_str(smart_quote_entity(el))
       end
 
+      MATH_LATEX_FILENAME = File.expand_path '../../data/math.json', __FILE__
+      MATH_LATEX = JSON.parse(File.read(MATH_LATEX_FILENAME, encoding: Encoding::UTF_8))
+      MATH_REPLACEMENTS = MATH_LATEX["replacements"]
+      MATH_COMBININGMARKS = MATH_LATEX["combiningmarks"]
+
+      def munge_latex(s)
+        MATH_REPLACEMENTS.each do |o, n|
+          s.gsub!(o, n)
+        end
+        MATH_COMBININGMARKS.each do |m, n|
+          re = /\\#{m[1..-1]}\{(\X)\}/
+          s.gsub!(re) { "#$1#{n}" }
+        end
+        s
+      end
+      # XXX: This is missing sup/sub support, which needs to be added
+
       def convert_math(el, indent, opts) # XXX: This is wrong
         el = el.deep_clone
         if el.options[:category] == :block
@@ -729,15 +990,24 @@ COLORS
               artwork_attr[md[1]] = v
             end
           end
-          result, _s = Open3.capture2("tex2mail -noindent -ragged -by_par -linelength=69", stdin_data: el.value);
+          result, err, _s = Open3.capture3("tex2mail -noindent -ragged -by_par -linelength=69", stdin_data: el.value);
           # warn "*** tex2mail not in path?" unless s.success? -- doesn't have useful status
+          capture_croak("tex2mail", err)
           "#{' '*indent}<figure#{el_html_attributes(el)}><artwork#{html_attributes(artwork_attr)}><![CDATA[#{result}#{result =~ /\n\Z/ ? '' : "\n"}]]></artwork></figure>\n"
 
         else
-          warn "*** no support for inline math in XML2RFCv2"
           type = 'spanx'
-          attrstring = el_html_attributes_with(el, {"style" => 'verb'})
-          "<#{type}#{attrstring}>#{escape_html(el.value, :text)}</#{type}>"
+          if $options.v3
+            type = 'contact'
+            result = munge_latex(el.value)
+            attrstring = el_html_attributes_with(el, {"fullname" => result.chomp, "asciiFullname" => ''})
+          else
+            warn "*** no support for inline math in XML2RFCv2"
+            type = 'spanx'
+            attrstring = el_html_attributes_with(el, {"style" => 'verb'})
+            content = escape_html(el.value, :text)
+          end
+          "<#{type}#{attrstring}>#{content}</#{type}>"
         end
       end
 
